#!/usr/bin/python3

################################################################################
# Anastasia Litina
# Eric Roca
# This script generates the maps displayed in the paper
# Paths are hardoced, please ensure that all the required data exists:
#   1) ./GIS/countries/countries.shp
#   2) ./GIS/merges_by_Anastasia
#   3) ./GIS/ethnicities_simplified.gpkg
#   4) ./GIS/solar_eclipses.gpkg
#   5) ./Data/data_for_maps.csv
#   6) ./GIS/Eclipses_at_cells.tif
#
# Ouput: several png files in the Latex folder
################################################################################

import pandas as pd
import geopandas as gpd
import matplotlib.pyplot as plt
import numpy as np
from mpl_toolkits.axes_grid1 import make_axes_locatable
import math
import rasterio
import rasterio.plot as rplot
import os
import matplotlib.cm as cm
import matplotlib.colors as colors


base_path = "./"
gis_path = os.path.join(base_path, "GIS")

def julian(year, month, day, verbose = False):
    if month == 1 or month == 12:
        yearp = year - 1
        monthp = month + 12
    else:
        yearp = year
        monthp = month

    if ((year < 1582) or
        (year == 1582 and month < 10) or
        (year == 1582 and month == 10 and day < 15)):
        B = 0
    else:
        A = math.trunc(year/100)
        B = 2 - A + math.trunc(A / 4.)

    if yearp < 0:
        C = math.trunc((365.25 * yearp) - 0.75)
    else:
        C = math.trunc(365.25 * yearp)

    D = math.trunc(30.6001 * (monthp + 1))

    julian = B + C + D + day + 1720994.5
    if verbose == True:
        print('Year:', year)
        print('Month:', month)
        print('Day:', day)

    return int(julian)

def prepare_maps(df, map_file, data_file):
    mapping = pd.read_csv(map_file, names = ['in_EA', 'in_FENSKE'], sep = ';')
    mapping = mapping.drop_duplicates()

    mapping['in_EA'] = mapping['in_EA'].str.upper()
    mapping['in_FENSKE'] = mapping['in_FENSKE'].str.upper()

    # Write everything in upper case to make matching easier
    df['mergename'] = df['mergename'].str.upper()
    df = df.merge(mapping, left_on = 'mergename', right_on = 'in_FENSKE', how = 'left')
    df['in_EA'] = df.in_EA.combine_first(df.mergename)

    print('Dissolving the geometries.')
    df = df.dissolve(by = 'in_EA')
    df = df.drop(columns = ['mergename', 'in_FENSKE'])
    df.reset_index(level = 0, inplace = True)

    data = pd.read_csv(data_file)
    data = data.rename(columns = {'navn' : 'in_EA'})
    data['in_EA'] = data['in_EA'].str.upper()

    df = df.merge(data, on = 'in_EA', how = 'inner')

    return df

def replace_legend_items(legend, mapping, title):
    for txt in legend.texts:
        for k,v in mapping.items():
            if txt.get_text() == str(k):
                txt.set_text(v)
    legend.set_title(title)


map_file = os.path.join(gis_path, "merges_by_Anastasia")
et_orig = os.path.join(gis_path, "ethnicities_simplified.gpkg")
data_file = os.path.join(base_path, "Data/data_for_maps.csv")
solar_ec = os.path.join(gis_path, "solar_eclipses.gpkg")
countries = os.path.join(gis_path, "Countries/countries.shp")

et = gpd.read_file(et_orig)
et_orig = gpd.read_file(et_orig)
df = prepare_maps(et, map_file, data_file)
ec = gpd.read_file(solar_ec)

df.loc[df.v90 == 0.0, 'v90'] = np.nan
df.loc[df.v90 == 8.0, 'v90'] = np.nan

saros_years = [-1239,-1221,-1203,-1185,-1167,-1149,-1131,-1113,-1095]
saros_months = [8,8,8,9,9,9,10,10,11]
saros_days = [7,18,28,9,19,30,11,22,1]

ec['saros'] = np.nan
ec['label'] = np.nan

ec['Date'] = ec['Date'].astype(int)

for i in zip(saros_years, saros_months, saros_days):
    ec.loc[ec.Date == julian(i[0], i[1], i[2]), 'saros'] = 97
    ec.loc[ec.Date == julian(i[0], i[1], i[2]), 'label'] = '{} {} {}'.format(i[0], i[1], i[2])
    


# World map
world = gpd.read_file(countries)
world = world[world.CONTINENT != "Antartica"]

def plot_v33(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    v33_dict = {1.0 : 'No levels', 2.0 : 'One level', 3.0 : 'Two levels', 4.0 : 'Three levels', 5.0 : 'Four levels'}
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 'v33', cmap = cmap, legend = True, missing_kwds = {'color' : 'lightgrey', 'label' : 'Missing data'}, categorical = True, ax = ax, edgecolor = 'black', linewidth = 0.8)
    replace_legend_items(ax.get_legend(), v33_dict, 'Jurisdictional hierarchy\nbeyond local community')
    plt.savefig(path, bbox_inches = 'tight')

def plot_v90(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    v90_dict = {1.0 : 'Absent, even at local level', 2.0 : 'Autonomous local\ncommunities', 3.0 : 'Peace groups transcending\nlocal communities', 4.0 : 'Minimal states', 5.0 : 'Little states', 6.0 : 'States'}
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 'v90', cmap = 'OrRd', legend = True, missing_kwds = {'color' : 'lightgrey', 'label' : 'Missing data'}, categorical = True, ax = ax, edgecolor = 'black', linewidth = 0.8)
    replace_legend_items(ax.get_legend(), v90_dict, 'Jurisdictional hierarchy\nbeyond local community')
    plt.savefig(path, bbox_inches = 'tight')

def plot_v66(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    v66_dict = {1.0 : 'Absent among freemen', 2.0 : 'Wealth distinctions', 3.0 : 'Elite', 4.0 : 'Dual', 5.0 : 'Complex'}
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 'v66', cmap = 'OrRd', legend = True, missing_kwds = {'color' : 'lightgrey', 'label' : 'Missing data'}, categorical = True, ax = ax, edgecolor = 'black', linewidth = 0.8)
    replace_legend_items(ax.get_legend(), v66_dict, 'Class stratification')
    plt.savefig(path, bbox_inches = 'tight')

def plot_tec(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    divider = make_axes_locatable(ax)
    cax = divider.append_axes('right', size = '1%', pad = 0.1)
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 'tec', cmap = 'OrRd', legend = True, missing_kwds = {'color' : 'lightgrey', 'label' : 'Missing data'}, categorical = False, ax = ax, edgecolor = 'black', linewidth = 0.8, cax = cax, legend_kwds = {'label' : 'Technological level', 'orientation' : 'vertical'})
    plt.savefig(path, bbox_inches = 'tight')

def plot_pop(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    s2_dict = {2.0 : '< 1p. / sq. mile', 3.0 : '1-5p. / sq. mile', 4.0 : '5-25 p. / sq. mile', 5.0 : '25-100 p. / sq. mile', 6.0 : '100-500 p. / sq. mile', 7.0 : '> 500 p. / sq. mile'}
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 's2', cmap = 'OrRd', legend = True, missing_kwds = {'color' : 'lightgrey', 'label' : 'Missing data'}, categorical = True, ax = ax, edgecolor = 'black', linewidth = 0.8)
    replace_legend_items(ax.get_legend(), s2_dict, 'Population density')
    plt.savefig(path, bbox_inches = 'tight')

def plot_eclipses(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="2%", pad=0.2)
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    df.plot(column = 'log_ec', cax = cax, cmap = cmap, legend = True, categorical = False, ax = ax, edgecolor = 'black', linewidth = 0.8)
    plt.savefig(path, bbox_inches = 'tight')

def plot_saros(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    myec = ec.loc[ec.saros == 97]
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    myec.plot(color = 'grey', ax = ax, edgecolor = 'darkgrey', linewidth = 0.8)
    plt.savefig(path, bbox_inches = 'tight')

def plot_random(num, path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    ec.sample(n = num, random_state = 31).plot(color = 'grey', ax = ax, edgecolor = 'darkgrey', linewidth = 0.8)
    plt.savefig(path, bbox_inches = 'tight')

ec_orig = [2413116, 2414327]
ec_hatch = {2413116 : '||', 2414327 : '//'}
et_orig = et_orig.loc[(et_orig.mergename == 'BORAN') | (et_orig.mergename == 'SAMBURU')]
xlim = ([et_orig.total_bounds[0]-4, et_orig.total_bounds[2]+4])
ylim = ([et_orig.total_bounds[1]-0.5, et_orig.total_bounds[3]+0.5])

def plot_zoom(path, bbox_inches = 'tight'):
    cmap = plt.get_cmap('Greys')
    fig = plt.figure(figsize = (30,20))
    ax = plt.gca()
    world.plot(ax = ax, color = 'white', edgecolor = 'black', linewidth = 0.5)
    et_orig.plot(ax = ax, color = 'orange', edgecolor = 'grey', linewidth = 0.8)
    et_orig['coords'] = et_orig['geometry'].apply(lambda x: x.representative_point().coords[:])
    et_orig['coords'] = [coords[0] for coords in et_orig['coords']]
    for idx, row in et_orig.iterrows():
        plt.annotate(s = row['mergename'], xy = row['coords'])
    for i in ec_orig:
        ec.loc[ec.Date == i].plot(ax = ax, color = 'lightgrey', edgecolor = 'grey', hatch = ec_hatch[i], legend = True)
    ax.set_xlim(xlim)
    ax.set_ylim(ylim)
    plt.savefig(path, bbox_inches = 'tight')
    
def plot_ec_distribution(path, eclipses_at_cell, ethnicities, world):
    # Open the raster dataset of eclipses at cell and get the colormap
    ec_cell = rasterio.open(eclipses_at_cell)
    cmap = plt.get_cmap('Greys')
    
    # Create a figure with a specific size and axes
    fig, ax = plt.subplots(figsize=(30, 20))
    
    # Plot the raster dataset with the given axes, using the specified colormap and the transformation
    # (i.e., the way in which the raster dataset is projected)
    rplot.show(ec_cell.read(1), ax=ax, transform=ec_cell.transform, cmap='Greys')
    
    # Plot the geometries of the world with the same axes, without filling the polygons and with a black color
    world.plot(ax=ax, facecolor='none', edgecolor='black', linewidth=0.5)
    
    # Plot the geometries of the ethnicities with the same axes, filling the polygons with a black color,
    # making them partially transparent and drawing them with a black color
    ethnicities.plot(ax=ax, facecolor='black', alpha=0.25, edgecolor='black')
    
    # Create a divider to make room for the colorbar and add it to the axes
    divider = make_axes_locatable(ax)
    cax = divider.append_axes("right", size="5%", pad=0.25)
    
    # Create a scalar mappable that maps the range of values of the raster dataset to the range of values
    # of the colormap, and add it to the colorbar
    fig.colorbar(cm.ScalarMappable(norm=colors.Normalize(vmin=np.nanmin(ec_cell.read(1)), vmax=np.nanmax(ec_cell.read(1))), cmap=cmap), cax=cax)
    
    # Save the figure to the given path, cropping the whitespace around the figure
    plt.savefig(path, bbox_inches="tight")

df['log_ec'] = np.log(df['total_ec'])
plot_eclipses(os.path.join(base_path, 'Latex/Figures/Total_Eclipses.png'))
#plot_v33(os.path.join(base_path, 'Latex/Figures/V33.png'))
#plot_v66(os.path.join(base_path, 'Latex/Figures/V66.png'))
#plot_v90(os.path.join(base_path, 'Latex/Figures/V90.png'))
#plot_tec(os.path.join(base_path, 'Latex/Figures/Technology.png'))
#plot_pop(os.path.join(base_path, 'Latex/Figures/Pop_Density.png'))

#plot_random(10, os.path.join(base_path, 'Latex/Figures/Coverage.png'))
plot_saros(os.path.join(base_path, 'Latex/Figures/Saros.png'))
#plot_zoom(os.path.join(base_path, 'Latex/Figures/Eclipses_Zoom.png'))
plot_ec_distribution(os.path.join(base_path, 'Latex/Figures/eclipses_distribution.png'), 
                     os.path.join(base_path, 'GIS/Eclipses_at_cells.tif'), 
                     et, 
                     world)
