#!/usr/bin/env python3

################################################################################
# Anastasia Litina
# Eric Roca
# This script generates generates Figure C.6, which consists of two images
# Paths are hardoced and the data is already loaded in the script
#
# Ouput: Discoveries_India.pdf
#        Discoveries_Greece.pdf           
################################################################################

import pandas as pd
import geopandas as gpd
import numpy as np
import math
import seaborn as sns
import os
import matplotlib.pyplot as plt

base_path = './'
gis_path  = os.path.join(base_path, 'GIS')

def jd_to_date(jd):
    """
    Convert Julian Day to date.
    
    Algorithm from 'Practical Astronomy with your Calculator or Spreadsheet', 
        4th ed., Duffet-Smith and Zwart, 2011.
    
    Parameters
    ----------
    jd : float
        Julian Day
        
    Returns
    -------
    year : int
        Year as integer. Years preceding 1 A.D. should be 0 or negative.
        The year before 1 A.D. is 0, 10 B.C. is year -9.
        
    month : int
        Month as integer, Jan = 1, Feb. = 2, etc.
    
    day : float
        Day, may contain fractional part.
        
    Examples
    --------
    Convert Julian Day 2446113.75 to year, month, and day.
    
    >>> jd_to_date(2446113.75)
    (1985, 2, 17.25)
    
    """
    jd = jd
    
    F, I = math.modf(jd)
    I = int(I)
    
    A = math.trunc((I - 1867216.25)/36524.25)
    
    if I > 2299160:
        B = I + 1 + A - math.trunc(A / 4.)
    else:
        B = I
        
    C = B + 1524
    
    D = math.trunc((C - 122.1) / 365.25)
    
    E = math.trunc(365.25 * D)
    
    G = math.trunc((C - E) / 30.6001)
    
    day = C - E + F - math.trunc(30.6001 * G)
    
    if G < 13.5:
        month = G - 1
    else:
        month = G - 13
        
    if month > 2.5:
        year = D - 4716
    else:
        year = D - 4715
        
    return year, month, day

def eclipses_discoveries(country_gpd, eclipse_gpd, discoveries_df, region, column):
    c = country_gpd.loc[country_gpd.NAME == region]
    d = discoveries_df.loc[discoveries_df['region'] == region]
    d = d.groupby('Year')[column].apply(sum).reset_index()
    start_year, end_year = (np.min(d['Year']), np.max(d['Year']))
    # Extract the eclipses for the given time frame and intersect with region boundaries
    # To intersect, we simplify the problem by exploding the region dataset
    # and intersecting row by row
    x = eclipse_gpd.loc[(eclipse_gpd['Year'] >= start_year) & (eclipse_gpd['Year'] <= end_year)].reset_index()
    c = c.assign(z = [range(0, len(x))]).explode("z").reset_index()
    x['Eclipses'] = x.intersects(c)
    d = d.merge(x.groupby('Year')['Eclipses'].apply(sum).reset_index(), on = 'Year')
    d = d.melt('Year', var_name='cols', value_name='Value')
    return d




eclipses_file = os.path.join(base_path, 'GIS/solar_eclipses.gpkg')
countries_file = os.path.join(gis_path, 'Countries/countries.shp')

# Data for discoveries soruced from:
# https://en.wikipedia.org/wiki/Timeline_of_scientific_discoveries#500_BC_%E2%80%93_1_BC
discoveries_file = os.path.join(base_path, 'Data/Discoveries.csv')

ec = gpd.read_file(eclipses_file)

# Convert the Julian date of an eclipse to the year corresponding to the beginning of the century
ec['Year'] = ec['Date'].apply(lambda x: jd_to_date(int(x))[0])
ec['Year'] = ec['Year'].apply(lambda x: math.floor(x/100)*100)

# Countries
countries = gpd.read_file(countries_file)

# Discoveries
dv = pd.read_csv(discoveries_file)
dv['Discoveries'] = 1
dv = dv.groupby(['region', 'year'])['Discoveries'].apply(sum).reset_index()
dv.rename(columns = {'year' : 'Year'}, inplace = True)

sns.set_theme()
sns.set_style("ticks")

regions = ["Greece", "India"]
for r in regions:
    out_file = os.path.join(base_path, 'Latex/Figures/Discoveries_{}.pdf'.format(r))
    d = eclipses_discoveries(countries, ec, dv, r, 'Discoveries')
    plt.figure()
    plot = sns.lineplot(data = d, x = 'Year', y = 'Value', hue = 'cols', palette = 'Greys')
    plot.legend_.set_title('')
    plot.set_title(r)
    plt.savefig(out_file)
