#!/usr/bin/env python3

################################################################################
# Anastasia Litina
# Eric Roca
# This script generates generates regression plots
# Paths are hardoced, please ensure that all the required data exists:
#   1) ./Data/database_plot_ea.dta
#
# Ouput: Figures 2, C.8. and C.9. 
################################################################################

import seaborn as sns
import pandas as pd
import os
import statsmodels.formula.api as smf # for regression model
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import numpy as np

base_path = "./"
save_path = os.path.join(base_path, "Latex/Figures/")
path_databases = os.path.join(base_path, "Data")


class sfg():

    def __init__(self, seaborngrid, fig,  subplot_spec):
        self.fig = fig
        self.sg = seaborngrid
        self.subplot = subplot_spec
        if (isinstance(self.sg, sns.axisgrid.FacetGrid) or isinstance(self.sg, sns.axisgrid.PairGrid)):
            self._movegrid()
        elif isinstance(self.sg, sns.axisgrid.JointGrid):
            self._movejointgrid()
        self._finalize()

    def _movegrid(self):
        """ Move PairGrid or Facetgrid """
        self._resize()
        n = self.sg.axes.shape[0]
        m = self.sg.axes.shape[1]
        self.subgrid = gridspec.GridSpecFromSubplotSpec(
            n, m, subplot_spec=self.subplot)
        for i in range(n):
            for j in range(m):
                self._moveaxes(self.sg.axes[i, j], self.subgrid[i, j])

    def _movejointgrid(self):
        """ Move Jointgrid """
        h = self.sg.ax_joint.get_position().height
        h2 = self.sg.ax_marg_x.get_position().height
        r = int(np.round(h / h2))
        self._resize()
        self.subgrid = (gridspec.GridSpecFromSubplotSpec(r + 1, r + 1,
                        subplot_spec=self.subplot))

        self._moveaxes(self.sg.ax_joint, self.subgrid[1:, :-1])
        self._moveaxes(self.sg.ax_marg_x, self.subgrid[0, :-1])
        self._moveaxes(self.sg.ax_marg_y, self.subgrid[1:, -1])

    def _moveaxes(self, ax, gs):
        # https://stackoverflow.com/a/46906599/4124317
        ax.remove()
        ax.figure = self.fig
        self.fig.axes.append(ax)
        self.fig.add_axes(ax)
        ax._subplotspec = gs
        ax.set_position(gs.get_position(self.fig))
        ax.set_subplotspec(gs)

    def _finalize(self):
        plt.close(self.sg.fig)
        self.fig.canvas.mpl_connect("resize_event", self._resize)
        self.fig.canvas.draw()

    def _resize(self, evt=None):
        self.sg.fig.set_size_inches(self.fig.get_size_inches())

def gen_regression(y, x, var):
    if x == "":
        return "{}~{}".format(y, var) 
    else:
        return "{}~{}+{}".format(y, x, var)

def va_plot(y, x, var, df, label_y, label_x, hue = None, save = None):
    d = df.dropna(subset = y)
    ry = smf.ols(gen_regression(y, "", var), data = d).fit()
    rx = smf.ols(gen_regression(x, "", var), data = d).fit()
    d["{}|X".format(label_y)] = df[y] - ry.predict(df)
    d["{}|X".format(label_x)] = df[x] - rx.predict(df)
    
    #tmp = sns.scatterplot(x = "{}|X".format(label_x), y = "{}|X".format(label_y), data = d, hue = hue, legend = False)
    #lg = tmp.legend()
    
    g0 = sns.jointplot(x = x, y = y, data = d, alpha = 0.5, hue = hue, legend = False, joint_kws={'color':'grey'}, marginal_kws={'color':'grey'})
    g0.set_axis_labels(label_x, label_y)
    g1 = sns.lmplot(x = "{}|X".format(label_x), y = "{}|X".format(label_y), data = d, ci = False, scatter = True, scatter_kws={'color' : 'grey'}, line_kws={'color' : 'black'})
    fig = plt.figure()
    gs = gridspec.GridSpec(1, 2)
    
    mg0 = sfg(g0, fig, gs[0])
    mg1 = sfg(g1, fig, gs[1])
    gs.tight_layout(fig)
    if save:
        plt.savefig(save, format = 'pdf', bbox_inches='tight')
    

sns.set()
sns.set_style("ticks")
g = sns.JointGrid()


reg = """bio1+bio12+clouds+distance_coast_km_log+distance_addis_km_log+
         distance_river_km_log+distance_volcano_km_log+distance_tectonic_km_log+
         rugavg+elevavg+malaravg+calavg+abs_lat+C(south)+C(mht_enc)+C(cont_enc)+C(tile_area)+C(v29)"""


common_vars_ea = ['bio1','bio12','clouds','distance_coast_km_log','distance_addis_km_log',
                       'distance_river_km_log','distance_tectonic_km_log','distance_volcano_km_log',
                       'rugavg','elevavg','calavg','malaravg','abs_lat','south','mht_enc',
                       'cont_enc','tile_area','v29']

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
va_plot('v30', 'eclipses_log', reg, df_ea, 'Sett. Patt.', 'Eclipses', save = save_path + "V30.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('v31', 'eclipses_log', reg, df_ea, 'Pop. Den.', 'Eclipses', save = save_path + "V31.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('v33', 'eclipses_log', reg, df_ea, 'Juris. Hier.', 'Eclipses', save = save_path + "V33.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('v66', 'eclipses_log', reg, df_ea, 'Class Start.', 'Eclipses', save = save_path + "V66.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
df_ea = df_ea.loc[(df_ea['v90'] > 0) & (df_ea['v90'] < 8)]
va_plot('v90', 'eclipses_log', reg, df_ea, 'Pol. Int.', 'Eclipses', save = save_path + "V90.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('explanation', 'eclipses_log', reg, df_ea, 'Eclipse Explanation', 'Eclipses', save = save_path + "Explanation.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('tasks', 'eclipses_log', reg, df_ea, 'Tasks', 'Eclipses', save = save_path + "Tasks.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('technology', 'eclipses_log', reg, df_ea, 'Technology', 'Eclipses', save = save_path + "Technology.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('writing', 'eclipses_log', reg, df_ea, 'Writing', 'Eclipses', save = save_path + "Writing.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('strategy', 'eclipses_log', reg, df_ea, 'Strat. Games', 'Eclipses', save = save_path + "Games.pdf")


df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('thinking_related', 'eclipses_log', reg, df_ea, 'Thinking rel.', 'Eclipses', save = save_path + "Thinking.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('calendar_related', 'eclipses_log', reg, df_ea, 'Calendar rel.', 'Eclipses', save = save_path + "Calendar.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('eclipse_related', 'eclipses_log', reg, df_ea, 'Eclipse rel.', 'Eclipses', save = save_path + "Eclipse.pdf")

df_ea = pd.read_stata(os.path.join(path_databases, "database_plot_ea.dta"))
df_ea.dropna(subset = common_vars_ea + ['eclipses_log'], inplace = True)
df_ea['v29'] = pd.Categorical(df_ea['v29'])
va_plot('curious_related', 'eclipses_log', reg, df_ea, 'Curious rel.', 'Eclipses', save = save_path + "Curious.pdf")
