#delimit ;
clear all;
clear matrix;
set maxvar 10000;
set varabbrev on;
global base_path       "./";
global path_stata       "${base_path}/Stata";
global path_tables      "${base_path}/Latex/Tables";
global path_figures     "${base_path}/Latex/Figures";
global path_data        "${base_path}/Data";

/******************************************************
* Dependencies
******************************************************/
ssc install estout;
ssc install acreg;
ssc install ranktest;
ssc install hdfe;
ssc install reghdfe;
ssc install ftools;
ssc install coefplot;
ssc install binscatter;
ssc install xlincom;
net install speccurve, from("https://raw.githubusercontent.com/martin-andresen/speccurve/master");

/******************************************************
* Sets of controls
******************************************************/

global myreg            "eclipses_log";
global distances        "distance_coast_km_log distance_addis_km_log distance_river_km_log 
                         distance_volcano_km_log distance_tectonic_km_log";
global geography        "rugavg elevavg malaravg calavg abs_lat south i.mht_enc";
global bioclimatic      "bio1 bio12 clouds lightning";
global area             "area_km";
global motifs           "eclipse_related calendar_related thinking_related cloud_related 
                         lightning_related rock_related sand_related white_related purple_related 
                         curious_related";
global motifs_religion  "religion_related pray_related religious_related";

/******************************************************
* Sets of dependent variables
******************************************************/

global outcomes_ea_3            "v31 v30";
global outcomes_ea_1            "v33 v66 v90 tasks technology strategy writing explanation";
global outcomes_seshat_3        "density";
global outcomes_seshat_1        "administrative_levels cc_hier cc_money cc_infra cc_writing cc_texts 
                                 cc_cal geometrical_measurements";
global outcomes_ashraf          "ln_CEtech_ ln_CITtech_ ln_trans_ ln_comm_ ln_indus_";
global outcomes_wikipedia       "science religious";
global placebo                  "cloud_related lightning_related rock_related sand_related 
                                 white_related purple_related";
global outcomes_folklore        "$motifs $motifs_religion $placebo";

global curiosity                "curious_related curiously_related curiosity_related";
global folklore_ols             "religion_related curious_related";
global think                    "thinking_related rethink_related think_related";
global drop_folklore            "curious_related think_related rethink_related thinking_related 
                                 thinker_related curiously_related";
global oprobit_regs             "v31 v30 v33 v66 v90 explanation";
global ols_regs                 "strategy writing technology religion_related";
global nbreg_regs               "tasks";
global probit_regs              "$curiosity purple_related";
global proxies_dev              "v33 v66 v90 administrative_levels cc_hier";
global proxies_h                "strategy writing explanation calendar_related thinking_related 
                                 cc_writing cc_texts cc_cal geometrical_measurements";
global proxies_t                "tasks technology cc_money cc_infra $outcomes_ashraf";
global proxies_w                "eclipse_related curious_related science";
global religion                 "v34 religion_related religious_related pray_related religious";
global table_area               "v31 v30 v33 v66 v90";
global main_folklore            "eclipse_related calendar_related thinking_related curious_related";
global poisson_regs             : list global(outcomes_folklore) - global(folklore_ols);

/******************************************************
* Sets of regressors
******************************************************/

/* Ethnographic Atlas
------------------------------------------------------*/

/* Continent fixed effects */
global EA0 "i.cont_enc";
/* Climate and geography */
global EA1 "$EA0 $bioclimatic $distances $geography i.tile_area";
/* Ethnic controls */
global EA2 "$EA1 i.v29";

/* Dictionary indicating the control we should add for each folklore concept
   see Section "Ethnic Level Data".
   This list does not include curious and pray because ConceptNet proposes essentially the same
   words as alternatives.
*/
global folklore_controls "
    rock_related        roll_related 
    purple_related      blue_related 
    cloud_related       rain_related
    thinking_related    dream_related
    calendar_related    date_related
    white_related       rice_related
    lightning_related   whistle_related
    sand_related        beach_related
    religious_related   church_related
    religion_related    believe_related";

/* Seshat
------------------------------------------------------*/
#delimit ;
global S0 "lightning clouds altitude ruggedness calories abs_lat tile_area_* quasipolity_* supra_* peak_date";

/* Augment the list by adding past values of population */
global S0 "${S0}";
/* Turchin indicates that regressions should account for past values of all outcomes variables */
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {;
    global S0 "${S0} p_`v'";
};
/* Turchin indicates that regressions should account for neighbour values of all outcomes variables */
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {;
    global S0 "${S0} n_`v'";
};

/******************************************************
* Clustering
*******************************************************/
/* Ethnographic Atlas
-------------------------------------------------------*/
global cluster_ea vce(cluster region);

/* Seshat
-------------------------------------------------------
We should cluster according to the most-removed predecessor used */
capture program drop cluster_seshat;
program cluster_seshat, rclass;
    if "`1'" == "acreg" {;
        return local r "spatial latitude(latitude) longitude(longitude) distcutoff(5000) 
                        id(`2') time(time) lag(1000) pfe1(`2') pfe2(time) hac";
    };
    if "`2'" == "reghdfe" {;
        return local r "cluster(`2')";
    };
end;

/******************************************************
* Labels
******************************************************/
global labels `"eclipses_log "Solar ec. (log)" 
                lunar_eclipses_log "Lunar ec. (log)" 
                eclipses_binary "Solar ec. (0/1)" 
                distance_volcano_km_log "Dist. volc. (log-km)" 
                distance_tectonic_km_log "Dist. fault (log-km)" 
                volcanoes "Volc. eruptions (log)" 
                volcanoes_binary "Volc. eruptions (0/1)""';

/******************************************************
* Programs
******************************************************/

/* Determine regression model and adjust it */
#delimit ;
capture program drop reg_model;
program reg_model, rclass;
    noisily di "`1'";
   /* Pass the outcome variable as argument */

   /* The sample should be limited when using V90, values 0 and 8 are for missing */
    if "`1'" == "v90" {;
        return local extras "if v90>0&v90<8";
    };
    else if (strpos("$outcomes_seshat_1", "`1'") > 0 | strpos("$outcomes_seshat_3", "`1'") > 0) & "`1'" != "writing" {;
            return local extras "if sd > 0";
    };
    else {;
        return local extras " ";
    };

    /* When the outcome is the number of tasks, control for the number of surveyed tasks */
    if "`1'" == "tasks" {;
        return local add_regressor "tasks_with_info";
    };

    /* When outcomes are from the folklore database we add as controls
         the number of published books
         the year of first publication
         the number of motifs
       Additionally, we clean the variables from terms that appear as related
       in ConceptNet
    */
    else if strpos("$outcomes_folklore", "`1'") {;
        local add_regressor "lnnmbr_title lnyear_firstpub lnmotifs_total";
        /* Use the global folklore_controls to determine the control we should add
           It is organized as a dictionary.
           First, we retrieve the position of the look-up word
           The next entry is the value associated to the searched key, so we add 1 to the previous index
       */
       if strpos("$folklore_controls", "`1'") {;
            local motif_position : list posof "`1'" in global(folklore_controls);
            local motif_position = `motif_position' + 1;
            local control : word `motif_position' of $folklore_controls;
            local add_regressor "`add_regressor' `control'";
        };
        return local add_regressor `add_regressor';
    };
    /* In the regressions using the Seshat, we control for total population throughout
       except (obviously) when the outcome variable is population density
   */
   else if strpos("$outcomes_seshat_1", "`1'") > 0 | strpos("$outcomes_seshat_3", "`1'") > 0 {;
        /* The previous expression also matches the Ethnographic Atlas variable `writing`.
           We create an ad-hoc rule to avoid adding `p_polity_population` to its regression.
        */
        if "`1'" == "writing" {;
            return local add_regressor " ";
        };
        else if "`1'" == "density" {;
            return local add_regressor " ";
        };
        else {;
            return local add_regressor "polity_population";
        };
    };
    else {;
        return local add_regressor " ";
    };

    /* Select the appropriate model */
    if strpos("$oprobit_regs", "`1'") {;
        return local method "oprobit";
        return local limit "iter(100)";
    };
    if strpos("$ols_regs", "`1'") {;
        return local method "reg";
        return local limit " ";
    };
    if strpos("$nbreg_regs", "`1'") {;
        return local method "nbreg";
        return local limit "iter(100)";
    };
    if strpos("$poisson_regs", "`1'") {;
        return local method "poisson";
        return local limit "iter(100)";
    };
    if strpos("$probit_regs", "`1'") {;
        return local method "probit";
        return local limit "iter(100)";
    };
end;

/* Add additional information to the regressions */
#delimit ;
capture program drop add_information;
program add_information;
    /* Pass the name of the locals and their values as arguments 
       Ex: add_information local_1 value_1 local_2 value_2 
    */
    tokenize `0';
    local n : word count `0';
    if mod(`n', 2) != 0 {;
        di "You should enter the information as local_1 value_1 local_2 value_2";
        exit;
    };
    forval i = 1/`n' {;
        if mod(`i', 2) == 1 {;
            local next = `i' + 1;
            qui estadd local ``i'' "``next''";
        };
        else {;
            continue;
        };
    };
end;

/* Store matrices to create other levels of standard errors */
capture program drop store_matrix;
qui program store_matrix;
    qui matrix regtab = r(table);
    qui matrix conleyse = regtab[2,1...];
    qui matrix conleyp = regtab[4,1...];
end;

/* Add stored matricies to estadd data
   We only use this program with the Seshat data.
   Because we run it as OLS, we also store the R2 as r2_p to have a consistent
   naming with respect to other regressions.
*/
capture program drop add_matrix;
program add_matrix;
   qui estadd matrix conleyse = conleyse;
   qui estadd matrix conleyp = conleyp;
   qui estadd scalar r2_p = e(r2);
end;

set scheme s2mono;
#delimit cr;

/* Summary stats */
do "$path_stata/7-Summary_Stats.do"

/* Clear the file with the values we export */
!echo "" >  "${base_path}/Latex/exported_values.txt"
eststo drop *
/* Regressions */
do "$path_stata/2-EA_Preparation.do"
do "$path_stata/2-EA.do"
do "$path_stata/3-Seshat.do"
do "$path_stata/4-Ashraf_Galor.do"
do "$path_stata/5-Wikipedia.do"
do "$path_stata/6-Tables.do"

eststo drop *
do "$path_stata/2-EA_Preparation.do"
do "$path_stata/2-EA_Robustness.do"
do "$path_stata/3-Seshat_Robustness.do"
do "$path_stata/4-Ashraf_Galor_Robustness.do"
do "$path_stata/5-Wikipedia_Robustness.do"
do "$path_stata/6-Tables_Robustness.do"

eststo drop *
do "$path_stata/2-EA_Preparation.do"
do "$path_stata/2-EA_Area.do"
do "$path_stata/3-Seshat_Robustness.do"
do "$path_stata/6-Tables_Area.do"

eststo drop *
do "$path_stata/2-EA_Preparation.do"
do "$path_stata/2-EA_Over_time.do"

eststo drop *
do "$path_stata/2-EA_Speccurve.do"
