/*
    Compute the effect of eclipses over time
    We do that by accumulating solar eclipses over time.
    This is, we begin using solar eclipses that occurred between
    -2000 and -1500, then we expand to cover -2000 to -1000, etc.
*/

/* Create shorter variable names so that we can handle these later */
local i 0
global short_outcomes_ea_3 ""
foreach v of global outcomes_ea_3 {
   gen ea3_`i' = `v'
   global short_outcomes_ea_3 "$short_outcomes_ea_3 ea3_`i'"
   local i = `i' + 1
}

local i 0
global short_outcomes_ea_1
foreach v of global outcomes_ea_1 {
   gen ea1_`i' = `v'
   global short_outcomes_ea_1 "$short_outcomes_ea_1 ea1_`i'"
   local i = `i' + 1
}

local i 0
global short_outcomes_folklore ""
foreach v of global main_folklore {
   gen fol_`i' = `v'
   global short_outcomes_folklore "$short_outcomes_folklore fol_`i'"
   local i = `i' + 1
}

/*
    Create the accumulated number of solar eclipses.
*/
gen cum_1500BC = log(1 + total_ec_2000BC_1500BC)
forval i=-1000(500)1500 {
    local ec_end = subinstr("`i'", "-", "", .)
    if `i' < 0 {
        local ec_end "`ec_end'BC"
    }

    local j = `i' - 500
    local ec_start = subinstr("`j'", "-", "", .)
    if `j' < 0 {
        local ec_start "`ec_start'BC"
    }

    gen cum_`ec_end' = log(exp(cum_`ec_start') + total_ec_`ec_start'_`ec_end')
}

/*
    Run separate regressions increasing the time span we consider
*/
forval i=-1500(500)1500 {
    local ec_end = subinstr("`i'", "-", "", .)
    if `i' < 0 {
        local ec_end "`ec_end'BC"
    }

    foreach v of global short_outcomes_ea_3 {
        /* Retrieve the original variable name because we are using the shortened names */
        local oldvar_pos : list posof "`v'" in global(short_outcomes_ea_3)
        local oldvar : word `oldvar_pos' of $outcomes_ea_3

        reg_model "`oldvar'"
        local method = r(method)
        local extras = r(extras)
        local limit  = r(limit)
        local add_regressor = r(add_regressor)

        qui `method' `v' cum_`ec_end' $EA2 `add_regressor' `extras', $cluster_ea `limit'
        qui eststo `v'_`ec_end'
    }

    foreach v of global short_outcomes_ea_1 {
        /* Retrieve the original variable name because we are using the shortened names */
        local oldvar_pos : list posof "`v'" in global(short_outcomes_ea_1)
        local oldvar : word `oldvar_pos' of $outcomes_ea_1

        reg_model "`oldvar'"
        local method = r(method)
        local extras = r(extras)
        local limit  = r(limit)
        local add_regressor = r(add_regressor)

        qui `method' `v' cum_`ec_end' $EA2 `add_regressor' `extras', $cluster_ea `limit'
        qui eststo `v'_`ec_end'
    }

    foreach v of global short_outcomes_folklore {
        /* Retrieve the original variable name because we are using the shortened names */
        local oldvar_pos : list posof "`v'" in global(short_outcomes_folklore)
        local oldvar : word `oldvar_pos' of $outcomes_folklore

        reg_model "`oldvar'"
        local method = r(method)
        local extras = r(extras)
        local limit  = r(limit)
        local add_regressor = r(add_regressor)

        qui `method' `v' cum_`ec_end' $EA2 `add_regressor' `extras', $cluster_ea `limit'
        qui eststo `v'_`ec_end'
    }
}

/* Plot the results */
set scheme s2mono

local mycoefplot "("
foreach v in $short_outcomes_ea_3 $short_outcomes_ea_1 {
    forval i=-1500(500)1500 {
        local ec_end = subinstr("`i'", "-", "", .)
        if `i' < 0 {
            local ec_end "`ec_end'BC"
        }

        local mycoefplot "`mycoefplot' `v'_`ec_end'"
    }
    local mycoefplot "`mycoefplot', aseq(`v') \"
}
local mycoefplot "`mycoefplot' )"


#delimit ;
coefplot `mycoefplot',
    keep(cum_*)  
    rename(^cum_([0-9]+[A-Z]*)$ = "Eclipses until \1", regex)
    xline(0)
    eqrename(
        ea3_0 = `""Population" "Density""'
        ea3_1 = `""Settlement" "Patterns""'
        ea1_0 = `""Jurisdictional" "Hierarchy""'
        ea1_1 = `""Political" "Integration""'
        ea1_2 = `""Class" "Stratification""'
        ea1_3 = "Tasks"
        ea1_4 = "Technology"
        ea1_5 = `""Games of" "Strategy""'
        ea1_6 = "Writing"
        ea1_7 = `""Eclipse" "Explanation""')
    levels(99 95 90) 
    ciopts(lwidth(3..) lcolor(*.2 *.4 *.6))
    ysize(15)
    ;
#delimit cr

translate @Graph "${path_figures}/EA_Over_time.pdf", name("Graph")

local mycoefplot "("
foreach v in $short_outcomes_folklore {
    local oldvar_pos : list posof "`v'" in global(short_outcomes_folklore)
    local oldvar : word `oldvar_pos' of $outcomes_folklore
    forval i=-1500(500)1500 {
        local ec_end = subinstr("`i'", "-", "", .)
        if `i' < 0 {
            local ec_end "`ec_end'BC"
        }

        local mycoefplot "`mycoefplot' `v'_`ec_end'"
    }
    local mycoefplot "`mycoefplot' ,aseq(`v') \"
}
local mycoefplot "`mycoefplot' )"

#delimit ;
coefplot `mycoefplot',
    keep(cum_*)  
    rename(^cum_([0-9]+[A-Z]*)$ = "Eclipses until \1", regex)
    xline(0)
    eqrename(
        fol_0 = `""Eclipse" "Related""'
        fol_1 = `""Calendar" "Related""'
        fol_2 = `""Thinking" "Related""'
        fol_3 = `""Curious" "Related""')
    levels(99 95 90) 
    ciopts(lwidth(3..) lcolor(*.2 *.4 *.6))
    ysize(15)
    ;
#delimit cr

translate @Graph "${path_figures}/Folklore_Over_time.pdf", name("Graph")
