/*  
    List all the covariates and its name, creating something similar
    to a dictionary.
    We use this for plotting.
*/
#delimit ;
local dict_covariates 
cont_enc Continent
bio1 Temperature
bio12 Precipitation
clouds Clouds
lightning Lightning
distance_coast_km_log Coast
distance_addis_km_log Addis
distance_river_km_log River
distance_volcano_km_log Volcano
distance_tectonic_km_log Fault
rugavg Ruggedness
elevavg Elevation
malaravg Malaria
calavg Calories
abs_lat AbsLat
south South
mht_enc Habitat
tile_area Area
v29 Crops;
#delimit cr

/* Reorder the main covariates, placing first the area, major habitat type and continent fixed effects */
#delimit ;
global EA2_sorted "i.tile_area i.cont_enc i.mht_enc 
                    south abs_lat bio1 bio12 clouds lightning 
                    distance_coast_km_log distance_addis_km_log distance_river_km_log distance_volcano_km_log distance_tectonic_km_log
                    rugavg elevavg malaravg calavg i.v29";
#delimit cr

/* Reorder the main outcomes to follow the same organization as in the tables */
global group_1 "v31 v30 v33 v66 v90"
global group_2 "strategy writing explanation calendar_related thinking_related"
global group_3 "tasks technology eclipse_related curious_related"
global groups "group_1 group_2 group_3"

local number_of_covariates : word count $EA2_sorted
foreach gr of global groups {
    eststo drop *
    foreach v of global `gr' {
        local covariates ""
        forval i=1/`number_of_covariates' {
            local add_covariate : word `i' of $EA2_sorted
            local covariates "`covariates' `add_covariate'"

            reg_model "`v'"
            local method = r(method)
            local extras = r(extras)
            local limit  = r(limit)
            local add_regressor = r(add_regressor)

            
            qui xi: `method' `v' eclipses_log `covariates' `add_regressor' `extras', $cluster_ea `limit'
            qui eststo `v'_`i'
            /* Retrieve the correct label from the dictionary
               According to the speccurve, we should store scalars named
               as the variables that active in a given regression with value 1
            */
            foreach z in `covariates' {
                /* If the covariate is passed as i.variable, remove the i. and search in the dictionary */
                if strpos("`z'", "i.") { 
                    local z = subinstr("`z'", "i.", "", .)
                    local position : list posof "`z'" in local(dict_covariates)
                    local position = `position' + 1
                    local label : word `position' of `dict_covariates'
                    qui estadd scalar `label' = 1
                }
                /* Otherwise just search in the dictionary */
                else {
                    local position : list posof "`z'" in local(dict_covariates)
                    local position = `position' + 1
                    local label : word `position' of `dict_covariates'
                    qui estadd scalar `label' = 1
                }
            }
            /* Add scalars with value 0 for the covariates that are not present in the regression
               What we do is create a global with all the covariates we use removing those that are present in a given regression
            */
            local not_present : list global(EA2_sorted) - local(covariates)
            foreach z in `not_present' {
                /* If the covariate is passed as i.variable, remove the i. and search in the dictionary */
                if strpos("`z'", "i.") {
                    local z = subinstr("`z'", "i.", "", .)
                    local position : list posof "`z'" in local(dict_covariates)
                    local position = `position' + 1
                    local label : word `position' of `dict_covariates'
                    qui estadd scalar `label' = 0
                }
                /* Otherwise just search in the dictionary */
                else {
                    local position : list posof "`z'" in local(dict_covariates)
                    local position = `position' + 1
                    local label : word `position' of `dict_covariates'
                    qui estadd scalar `label' = 0
                }
            }
        }
    }
    /* We have to generate a "panel" that will contain the regression names and the associated labels */
    local panel ""
    foreach z of global EA2_sorted {
        /* If the covariate is passed as i.variable, remove the i. and search in the dictionary */
        if strpos("`z'", "i.") {
            local z = subinstr("`z'", "i.", "", .)
            local position : list posof "`z'" in local(dict_covariates)
            local position = `position' + 1
            local label : word `position' of `dict_covariates'
        }
        /* Otherwise just search in the dictionary */
        else {
            local position : list posof "`z'" in local(dict_covariates)
            local position = `position' + 1
            local label : word `position' of `dict_covariates'
        }
        local panel "`panel' `label'"
    }

    /* Place xlines at correct intervals to separate the dependent variables */
    local all_models "" 
    foreach v of global `gr' {
        local all_models "`all_models' `v'"
    }
    local num_models : word count `all_models'
    local num_models  = `num_models' - 1
    local num_covariates : word count $EA2
    local xlines ""
    forval i=1/`num_models' {
        local val = `i'*`num_covariates'
        local xlines "`xlines' `val'"
    }
    di "All models `all_models'"

    speccurve, param(eclipses_log) panel(`panel') sort(none) graphopts(xline(`xlines') ysize(30) ytitle("Coefficient on eclipses") legend(off))
    graph export "${path_figures}/`gr'.pdf", as(pdf) replace
}
