/* This do-file produces the results when we employ the Seshat database
   It contains two main parts:
   1) Analysis using the original polity name as fixed effect
   2) Analysis using the most-removed ancestor as fixed effect

   The second part requires further processing because, for each polity, we need
   to determine its ancestor and compute the lagged value of the variables/
*/

/*************************************************** 
   Part 1, Analysis using the original polity name
****************************************************/

do "$path_stata/3-Seshat_Preparation.do"

/* Generate the past value of each variable we use.
   These are named p_`var`
*/
qui gen sorter = _n
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    qui gen p_`v' = .
    /* Order the database by name and time */
    qui by original_culture_name (time sorter), sort: replace p_`v' = `v'[_n-1] if time != time[_n-1]
    qui by original_culture_name (time sorter), sort: replace p_`v' = `v'[_n-2] if time == time[_n-1]
}

/* Regressions should account for the fact that the current value
   of a given variable for polity `i' is likely to be influenced by the value of this variable
   in neighbouring regions.
   We create a series of variables named n_`var` that keep track of the value in neighbouring regions.
   We apply an exponential decay based on distance.
*/
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    capture drop n_`v' 
    gen n_`v' = .
}

/* Trigonometric operations are in radians, we need the value of pi */ 
qui scalar p = 3.1416/180

/* The exponenial decay we use is continuous.
   We select a value for the exponenial form such that a society that is 5000km away has a weight of only 1%.
*/
local dist_cutoff = 1085
qui capture drop distance
qui gen distance = .
qui levels original_culture_name, local(pols)
foreach pol of local pols {
    qui levelsof period_start if original_culture_name == "`pol'", local(years)
    foreach year of local years {
        /* Compute the distance between polity `i' and its neighbours.
           We only do so if the year coincides.
        */
        qui capture drop distance
        qui gen distance = .
        qui capture drop t_var
        qui gen t_var = .
        qui sum latitude if original_culture_name == "`pol'" & period_start == `year'
        qui local lat_orig = r(mean)
        qui sum longitude if original_culture_name == "`pol'" & period_start == `year'
        qui local lon_orig = r(mean)
        qui replace distance = asin(sqrt(0.5 - cos((latitude - `lat_orig')*p)/2 + cos(`lat_orig' * p) * cos(latitude * p) * (1 - cos((longitude - `lon_orig') * p))/2))*12742 if original_culture_name != "`pol'" & period_start == `year'
        foreach var in $outcomes_seshat_3 $outcomes_seshat_1 {
            /* Compute the neighbour effect using an exponential decay function */
            qui replace t_var = p_`var' * exp(- distance / `dist_cutoff')
            qui sum t_var
            qui replace n_`var' = r(mean) if original_culture_name == "`pol'" & period_start == `year'
        }
    }
}

/* This section runs the regressions corresponding to 
   1) Analysis using the original polity name as fixed effect
*/
qui local id "id"
qui local conley_seshat "spatial latitude(latitude) longitude(longitude) distcutoff(5000) id(`id') time(time) lag(1000) pfe1(`id') pfe2(time) hac"

foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    reg_model "`v'"
    local method = r(method)
    local extras = r(extras)
    local limit  = r(limit)
    local add_regressor = r(add_regressor)

    capture drop sd
    qui by `id', sort: egen sd = sd(`v')
    
    /* Some regressions cannot be estimated.
       To check, we first run both acreg and reghdfe with capture and only
       when the two are successful we store the results
    */
    qui capture acreg `v' eclipses volcanoes $S0 `add_regressor' `extras', `conley_seshat'
        if _rc == 0 {
            qui capture reghdfe `v' eclipses volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
            if _rc == 0 {
                /* Both are successful */
                qui acreg `v' eclipses volcanoes $S0 `add_regressor' `extras', `conley_seshat'
                store_matrix
                qui reghdfe `v' eclipses volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
                eststo `v'_1
                add_information fe "Polity" tim "Yes" con "Yes" ethnic "No" weights "No" geo "Yes" 
                add_matrix
                !echo "beta_`v'_eclipses_1,`z'" >> "${base_path}/Latex/exported_values.txt"
            }
        }
}

/*************************************************** 
   Part 2, Analysis using the most-removed ancestor
           as fixed effect
****************************************************/

clear
do "$path_stata/3-Seshat_Preparation.do"

/* Generate the past value of each variable we use.
   These are named p_`var`
*/
drop if missing(genealogy_1)

gen acronym = substr(temperoculture, 1, max(strpos(temperoculture, "+"), strpos(temperoculture, "-"))-1)
sort genealogy_1 time
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    qui capture gen p_`v' = .
    forval i=1/`=_N' {
        local a=acronym[`i']
        local t=period_end[`i']
        qui sum period_end if acronym=="`a'" & period_end < `t'
        qui sum `v' if acronym == "`a'" & period_end == r(max)
        if r(N) > 0 {
            qui replace p_`v' = r(mean) in `i'
        }
        else { 
            qui sum period_end if genealogy_1 == genealogy_1[`i'] & period_end < period_end[`i']
            local m = r(max)
            qui sum `v' if genealogy_1 == genealogy_1[`i'] & period_end == `m'
            if r(N) > 0 {
                qui replace p_`v' = r(mean) in `i'
            }
        }
    }
}

/* Regressions should account for the fact that the current value
   of a given variable for polity `i' is likely to be influenced by the value of this variable
   in neighbouring regions.
   We create a series of variables named n_`var` that keep track of the value in neighbouring regions.
   We apply an exponential decay based on distance.
*/
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    capture drop n_`v' 
    gen n_`v' = .
}

/* Trigonometric operations are in radians, we need the value of pi */ 
qui scalar p = 3.1416/180

/* The exponenial decay we use is continuous.
   We select a value for the exponenial form such that a society that is 5000km away has a weight of only 1%
*/
local dist_cutoff = 1085
qui capture drop distance
qui gen distance = .
qui levels original_culture_name, local(pols)
foreach pol of local pols {
    qui levelsof period_start if original_culture_name == "`pol'", local(years)
    foreach year of local years {
        /* Compute the distance between polity `i' and its neighbours.
           We only do so if the year coincides.
        */
        qui capture drop distance
        qui gen distance = .
        qui capture drop t_var
        qui gen t_var = .
        qui capture drop the_total
        qui sum latitude if original_culture_name == "`pol'" & period_start == `year'
        local lat_orig = r(mean)
        qui sum longitude if original_culture_name == "`pol'" & period_start == `year'
        local lon_orig = r(mean)
        qui replace distance = asin(sqrt(0.5 - cos((latitude - `lat_orig')*p)/2 + cos(`lat_orig' * p) * cos(latitude * p) * (1 - cos((longitude - `lon_orig') * p))/2))*12742 if original_culture_name != "`pol'" & period_start == `year'
        foreach var in $outcomes_seshat_3 $outcomes_seshat_1 {
            /* Compute the neighbour effect using an exponential decay function */
            qui replace t_var = p_`var' * exp(- distance / `dist_cutoff')
            qui sum t_var
            qui replace n_`var' = r(mean) if original_culture_name == "`pol'" & period_start == `year'
        }
    }
}

/* This section runs the regressions corresponding to 
   2), Analysis using the most-removed ancestor as fixed effect
*/
local id "genealogy_id"
local conley_seshat "spatial latitude(latitude) longitude(longitude) distcutoff(5000) id(`id') time(time) lag(1000) pfe1(`id') pfe2(time) hac"

foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    reg_model "`v'"
    local method = r(method)
    local extras = r(extras)
    local limit  = r(limit)
    local add_regressor = r(add_regressor)

    capture drop sd
    qui by `id', sort: egen sd = sd(`v')
    replace sd = 2

    qui acreg `v' eclipses volcanoes $S0 `add_regressor' `extras', `conley_seshat'
        store_matrix
    qui reghdfe `v' eclipses volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
       qui eststo `v'_2
       add_information fe  "Polity" tim "Yes" con "Yes" ethnic "No" weights "No" geo "Yes"
       add_matrix
       *qui local z : di %5.2f _b[eclipses]
       *!echo "beta_`v'_eclipses_2,`z'" >> "${base_path}/Latex/exported_values.txt"
}
