clear
import delimited "seshat_genealogy.csv", varnames(1)
duplicates drop temperoculture, force
tempfile seshat_genealogy
save "seshat_genealogy", replace
clear

/* Read additional data */
import delimited "seshat_additional_data.csv"
keep temperoculture eclipses eclipses_lunar volcanoes area_geom lightning ruggedness calories altitude clouds latitude longitude
duplicates drop temperoculture, force
tempfile seshat_additional_data
save "seshat_additional_data", replace

/* Read the Seshat database */
clear
import delimited "seshat.csv"
drop latitude
drop longitude
merge 1:1 temperoculture using "seshat_additional_data", gen(_merge_geom)
keep if _merge_geom == 3

merge 1:1 temperoculture using "seshat_genealogy", gen(_genealogy)

* Generate log(calories/area)
replace calories = calories/area_geom
replace calories = log(1+calories)

qui replace eclipses  = log(1 + eclipses)
qui replace volcanoes = log(1 + volcanoes)
qui replace lightning = log(1 + lightning)
qui replace clouds    = log(1 + clouds)
qui gen abs_lat       = abs(latitude)

/* We also have a variable named calendar in the folklore.
   We rename it for the Seshat to avoid confusion.
*/
rename calendar cc_cal


/* Replace some typos */
replace original_culture_name = "Byzantine Empire" if original_culture_name == "Bzyantine Empire"
replace original_culture_name = "Diina" if original_culture_name == "diina"

* Setup a local variable with the relevant variables
* Following https://doi.org/10.21237/C7clio9137696 we will lag them and introduce the lagged variables as regressors
* note: for the moment we are not dealing with spatial correlation

/* Create fixed effects
   1) at the original cultural name level
*/
   encode original_culture_name , gen(id)
/*
   2) at the most-removed ancestor level */
   encode genealogy_1 , gen(genealogy_id)

/* Fixed effects must be positive. Transform them */
qui sum period_start
gen time = period_start + abs(r(min)) + 100 

/* Generate population density */
gen density = log(polity_population / area_geom)
drop if missing(eclipses)

/* Drop some observations that are likely to bias the results:
   European empires in the Americas or Asia
*/
drop if original_culture_name == "British Colonial Period"
drop if original_culture_name == "East India Company"
drop if original_culture_name == "Mysore British India"
drop if original_culture_name == "Spanish Habsburgs"
drop if original_culture_name == ""
drop if temperoculture == "FrBurbL+1700"                /* French Empire in Mississippi */
drop if temperoculture == "FrBurbL+1800"                /* French Empire in Mississippi */
drop if temperoculture == "CahokiaExtra+1800"           /* French Empire in Mississippi */
drop if original_culture_name == "Kingdom of Norway"    /* Norway, but the eclipse data corresponds to Iceland */

/* Consider time until 1800 */
drop if period_start >= 1900

/* Drop duplicated entries */
duplicates drop time id area_geom, force
duplicates drop time id preceding_quasipolity, force

/* Create area deciles (akin to the EA standard case) */
xtile tile_area = area_geom, nq(10)
tab tile_area, gen(tile_area_)

qui replace geometrical_measurements = 0 if geometrical_measurements < 0

/* Log-transform the variables.
   Note that population density is already in logs
*/
foreach v in polity_population $outcomes_seshat_1 $outcomes_seshat_3 {
    if "`v'" == "density" {
        continue
    }
    replace `v' = log(1+`v')
}

/* The variable stating how a polity relates to its predecessor includes
   levels that are essentially the same, but with different names.
   We manually correct this.
*/
qui gen relationship = ""
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "Continuation"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "Continuity"
qui replace relationship = "change" if relationship_to_preceding_quasip == "Regime change"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "Same cultural group"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "contination"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "continuation"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "continuity"
qui replace relationship = "continuation" if relationship_to_preceding_quasip == "continunity"
qui replace relationship = "assimilation" if relationship_to_preceding_quasip == "cultural assimilation"
qui replace relationship = "elite migration" if relationship_to_preceding_quasip == "elite migration"
qui replace relationship = "revolt" if relationship_to_preceding_quasip == "indigenous revolt"
qui replace relationship = "population migration" if relationship_to_preceding_quasip == "population migration"
qui replace relationship = "vassalage" if relationship_to_preceding_quasip == "vassalage"
qui tab relationship, gen(quasipolity_)

/* We do the same for supra-polity relationships */
qui gen supra = ""
qui replace supra = "none" if suprapolity_relations == "0.0"
qui replace supra = "alliance" if suprapolity_relations == "alliance"
qui replace supra = "allegiance" if suprapolity_relations == "nominal"
qui replace supra = "allegiance" if suprapolity_relations == "nominal allegiance"
qui replace supra = "none" if suprapolity_relations == "none"
qui replace supra = "union" if suprapolity_relations == "personal union"
qui replace supra = "vassalage" if suprapolity_relations == "vassalage"
qui tab supra, gen(supra_)

/* Replace the peak date with the closes century */
qui replace peak_date = 100*round(peak_date / 100)
