clear
do "$path_stata/3-Seshat_Preparation.do"

/* We use only the most-removed predessor fixed effects */

/* Generate the past value of each variable we use.
   These are named p_`var`
*/
drop if missing(genealogy_1)

gen acronym = substr(temperoculture, 1, max(strpos(temperoculture, "+"), strpos(temperoculture, "-"))-1)
sort genealogy_1 time
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    qui capture gen p_`v' = .
    forval i=1/`=_N' {
        local a=acronym[`i']
        local t=period_end[`i']
        qui sum period_end if acronym=="`a'" & period_end < `t'
        qui sum `v' if acronym == "`a'" & period_end == r(max)
        if r(N) > 0 {
            qui replace p_`v' = r(mean) in `i'
        }
        else { 
            qui sum period_end if genealogy_1 == genealogy_1[`i'] & period_end < period_end[`i']
            local m = r(max)
            qui sum `v' if genealogy_1 == genealogy_1[`i'] & period_end == `m'
            if r(N) > 0 {
                qui replace p_`v' = r(mean) in `i'
            }
        }
    }
}

/* Regressions should account for the fact that the current value
   of a given variable for polity `i' is likely to be influenced by the value of this variable
   in neighbouring regions.
   We create a series of variables named n_`var` that keep track of the value in neighbouring regions.
   We apply an exponential decay based on distance.
*/
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    qui capture drop n_`v' 
    qui gen n_`v' = .
}

/* Trigonometric operations are in radians, we need the value of pi */ 
qui scalar p = 3.1416/180

/* The exponenial decay we use is continuous.
   We select a value for the exponenial form such that a society that is 5000km away has a weight of only 1%
*/
local dist_cutoff = 1085 
capture drop distance
qui gen distance = .
qui levels original_culture_name, local(pols)
foreach pol of local pols {
    qui levelsof period_start if original_culture_name == "`pol'", local(years)
    foreach year of local years {
        /* Compute the distance between polity `i' and its neighbours.
           We only do so if the year coincides.
        */
        qui capture drop distance
        qui gen distance = .
        qui capture drop t_var
        qui gen t_var = .
        qui capture drop the_total
        qui sum latitude if original_culture_name == "`pol'" & period_start == `year'
        local lat_orig = r(mean)
        qui sum longitude if original_culture_name == "`pol'" & period_start == `year'
        local lon_orig = r(mean)
        qui replace distance = asin(sqrt(0.5 - cos((latitude - `lat_orig')*p)/2 + cos(`lat_orig' * p) * cos(latitude * p) * (1 - cos((longitude - `lon_orig') * p))/2))*12742 if original_culture_name != "`pol'" & period_start == `year'
        foreach var in $outcomes_seshat_3 $outcomes_seshat_1 {
            /* Compute the neighbour effect using an exponential decay function */
            qui replace t_var = p_`var' * exp(- distance / `dist_cutoff')
            qui sum t_var
            qui replace n_`var' = r(mean) if original_culture_name == "`pol'" & period_start == `year'
        }
    }
}

/* Regressions */
local id "genealogy_id"
local conley_seshat "spatial latitude(latitude) longitude(longitude) distcutoff(5000) id(`id') time(time) lag(1000) pfe1(`id') pfe2(time) hac"

/* Adding lunar eclipses */
foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    reg_model "`v'"
    local method = r(method)
    local extras = r(extras)
    local limit  = r(limit)
    local add_regressor = r(add_regressor)
    forval i=0/1 {
        if `i' == 0 {
            local variables "eclipses_lunar "
        }
        if `i' == 1 { 
            local variables "eclipses eclipses_lunar "
        }

        capture drop sd
        qui by `id', sort: egen sd = sd(`v')
        replace sd = 2

        * Regressions with time and polity fixed effects
        qui acreg `v' `variables' volcanoes $S0 `add_regressor' `extras', `conley_seshat'
            store_matrix
        qui reghdfe `v' `variables' volcanoes $S0 `extra', absorb(i.time i.`id') cluster(`id')
            eststo `v'_r`i'
            add_information fe  "Polity" tim "Yes" con "Yes" ethnic "No" weights "No" geo "Yes"
            add_matrix
    }
}

* Regressions including eclipses/area as additional regressor
* To avoid endogeneity, these regressions must control for the number of eclipses and area
/*
    Use the number of eclipses per unit of area as regressor.
    To avoid endogeneity, this regression should also control for area and the number of eclipses
    We create these variables.

    To gauge the effect of solar eclipses, we compute the marginal effect they have.
*/
qui replace area_geom = area_geom/10^8
capture drop inv_area
capture drop num_eclipses
capture drop inter
gen inv_area = 1/area_geom
gen num_eclipses = exp(eclipses) - 1
gen inter = num_eclipses*inv_area

local id "genealogy_id"
local conley_seshat "spatial latitude(latitude) longitude(longitude) distcutoff(5000) id(`id') time(time) lag(1000) pfe1(`id') pfe2(time) hac"

foreach v in $outcomes_seshat_3 $outcomes_seshat_1 {
    reg_model "`v'"
    local method = r(method)
    local extras = r(extras)
    local limit  = r(limit)
    local add_regressor = r(add_regressor)

    capture drop sd
    qui by `id', sort: egen sd = sd(`v')
    replace sd = 2

    /* Regressions with time and polity fixed effects */
    qui acreg `v' num_eclipses inter area_geom volcanoes $S0 `add_regressor' `extras', `conley_seshat'
        store_matrix
    qui reghdfe `v' c.num_eclipses inter area_geom volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
        eststo `v'_i
        add_information fe  "Polity" tim "Yes" con "Yes" ethnic "No" weights "No" geo "Yes"
        add_matrix

    /* 
        Marginal effects
        First, re-run a reghdfe to obtain the sample
        acreg does not correctly report it
    */
    qui reghdfe `v' c.num_eclipses inter area_geom volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
	capture qui drop in_sample
	qui gen in_sample = e(sample)

    preserve
    qui keep if in_sample == 1
    qui acreg `v' c.num_eclipses inter area_geom volcanoes $S0 `add_regressor' `extras', `conley_seshat'
    qui sum inv_area if in_sample
    qui local mean_inv_area = r(mean)
    qui xlincom _b[c.num_eclipses] + _b[inter] * `mean_inv_area', df(1000) post
        store_matrix

    qui reghdfe `v' c.num_eclipses inter area_geom volcanoes $S0 `add_regressor' `extras', absorb(i.time i.`id') cluster(`id')
    qui sum inv_area if e(sample)
    qui local mean_inv_area = r(mean)
    qui xlincom _b[c.num_eclipses] + _b[inter] * `mean_inv_area', df(1000) post
        qui eststo `v'_mg
        add_information fe  "Polity" tim "Yes" con "Yes" ethnic "No" weights "No" geo "Yes"
        add_matrix
	qui sum `v' if e(sample)
	qui estadd scalar avg = r(mean)
	qui estadd scalar std = r(sd)
    restore
}
