clear
import delimited "people_wikipedia_eclipses.csv"

/* Fill missing number of eclipses and volcanoes with zero */
replace eclipses = 0 if eclipses == .
replace volcanoes = 0 if volcanoes == .

/* Convert these variables to binary */
gen eclipses_binary = (eclipses > 0)
gen volcanoes_binary = (volcanoes > 0)

/*
    Almost everybody is exposed to one lunar eclipse in 10 years
    Instead, we control for the number of total lunar eclipses observed
*/
replace eclipses_lunar = 0 if eclipses_lunar == .
gen eclipses_lunar_binary = (eclipses_lunar > 0)
gen eclipses_lunar_log = log(1 + eclipses_lunar)

/*
    Science and religion are encoded as chars
    Create numeric values for these
*/
gen sc = .
replace sc = 1 if science == "True"
replace sc = 0 if science == "False"
drop science
rename sc science

gen rel = .
replace rel = 1 if religious == "True"
replace rel = 0 if religious == "False"
drop religious
rename rel religious

/*
    Generate century dummies
    1st January -2000 is 990923 in Julian
*/
scalar first_julian = 990923
gen century = floor((date-first_julian)/(100*365))

encode name, gen(country)

/* Drop duplicated people: they appear when they have multiple occupations */
duplicates drop person, force

/* We can use pweights to rebalance the sample */
by century, sort: gen weight_1 = 1/_N

/*
    We can also use world population estimates from McEvecy and Jones (1978)
    Before the year 0, we only have data for -2000, -1000 and -200, interpolate in between.
    https://en.wikipedia.org/wiki/Estimates_of_historical_world_population
*/
capture drop world_pop
capture drop weight_2
gen world_pop = .
replace world_pop = 72 if century == 1
replace world_pop = 115 if century == 11

/* Consider exponential growth for population: pop_11 = pop_1 exp(g*t) */
local g=ln(115/72)/(11-1) /* g = 0.0468 */

forval i=2/10 {
    replace world_pop = 27*exp(`g'*(`i'-1)) if century == `i'
}
replace world_pop = 188 if century == 21

local g=ln(188/115)/(21-11)
forval i=12/21 {
    replace world_pop = 115*exp(`g'*(`i'-11)) if century == `i'
}

replace world_pop = 195 if century == 22
replace world_pop = 202 if century == 23
replace world_pop = 205 if century == 24
replace world_pop = 209 if century == 25
replace world_pop = 210 if century == 26
replace world_pop = 213 if century == 27
replace world_pop = 226 if century == 28
replace world_pop = 240 if century == 29
replace world_pop = 269 if century == 30
replace world_pop = 295 if century == 31
replace world_pop = 353 if century == 32
replace world_pop = 393 if century == 33
replace world_pop = 392 if century == 34
replace world_pop = 390 if century == 35
replace world_pop = 461 if century == 36
replace world_pop = 554 if century == 37
replace world_pop = 603 if century == 38
by century, sort: gen weight_2 = world_pop/_N

/* Regressions with lunar eclipses adn lunar and solar eclipses */
foreach v of global outcomes_wikipedia {
    forval i=0/1 {
        if `i' == 0 {
            local variables "eclipses_lunar_log"
        }
        if `i' == 1 {
            local variables "eclipses_binary eclipses_lunar_log"
        }
        forval j=1/2 {
            qui reghdfe `v' `variables' volcanoes_binary [pw=weight_`j'], absorb(country century) cluster(country#century)
            qui eststo `v'_`j'_rob_`i'
            add_information fe "City" tim "Yes" geo "No" ethnic "No"
            qui estadd scalar r2_p = e(r2)
            if `j' == 1 {
                add_information weights "Sample"
            }
            if `j' == 2 {
                add_information weights "Population"
            }
        }
    }
}
